@@ -1,5 +1,5 @@
 from ._version import __version__
 from ._py7za import Py7za, arg_split
-from ._nice_size import nice_size
+from ._nice_size import nice_size, size_to_int
 from ._asyncio_pool import AsyncIOPool
 from ._cpu_count import available_cpu_count
